#!/usr/bin/python3

# Copyright (c) 2023 ITRS Group Limited

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

# Change History
# 2023-03-22 (Howard Lee) Initial release

import os
import re
import sys
import base64
import urllib.request
import urllib.error
import html
import json

gateway_hostname='http://localhost'
gateway_username='restuser'
gateway_password='Ap1P@ssw0rd'

env_gateway = os.environ.get('_GATEWAY', '')
if env_gateway == '':
  print('Could not obtain environment variable _GATEWAY.')
  print('This script should be run in a gateway action.')
  sys.exit(1)

env_variablepath = os.environ.get('_VARIABLEPATH', '')
if env_variablepath == '':
  print('Could not obtain environment variable _VARIABLEPATH.')
  print('This script should be run in a gateway action.')
  sys.exit(1)

env_probe = os.environ.get('_PROBE', '')
env_netprobehost = os.environ.get('_NETPROBE_HOST', '')
env_managedentity = os.environ.get('_MANAGED_ENTITY', '')
env_sampler = os.environ.get('_SAMPLER', '')
env_pluginname = os.environ.get('_PLUGINNAME', '')
env_dataview = os.environ.get('_DATAVIEW', '')
env_action = os.environ.get('_ACTION', '')
env_headline = os.environ.get('_HEADLINE', '')
env_column = os.environ.get('_COLUMN', '')
env_rowname = os.environ.get('_ROWNAME', '')
env_value = os.environ.get('_VALUE', '')
env_severity = os.environ.get('_SEVERITY', '')

print("""<!DOCTYPE html>
<HTML lang="en">
<TITLE>Geneos Alert</TITLE>
<style>
table { border-width: 1px 1px 1px 1px; border-style: solid; border-spacing: 0; }
th { border-width: 1px 1px 1px 1px; border-style: solid; padding: 4px }
td { border-width: 1px 1px 1px 1px; border-style: solid; padding: 4px }
</style>""")

message = """
This is an automatically generated mail from Geneos Gateway: {_GATEWAY}
<P>
Action "{_ACTION}" is being fired against Geneos DataItem {_VARIABLEPATH}
<P>
The dataitem value is "{_VALUE}" and its severity is {_SEVERITY}
""".format(_GATEWAY=env_gateway, _ACTION=env_action, _VARIABLEPATH=env_variablepath, _VALUE=env_value, _SEVERITY=env_severity)
print(message)

print('<TABLE>')
print('<TR><TD>Gateway</TD><TD>'+env_gateway+'</TD></TR>')
print('<TR><TD>Probe</TD><TD>'+env_probe+' (Host: '+env_netprobehost+')</TD></TR>')
print('<TR><TD>Managed Entity</TD><TD>'+env_managedentity+'</TD></TR>')
print('<TR><TD>Sampler</TD><TD>'+env_sampler+' (Plugin: '+env_pluginname+')</TD></TR>')
print('<TR><TD>Dataview</TD><TD>'+env_dataview+'</TD></TR>')

if env_headline != '':
  if env_severity == 'CRITICAL':
    print('<TR><TD><B>'+env_headline+' (Headline)</B></TD><TD style="background-color:red">'+env_value+'</TD></TR>')
  elif env_severity == 'WARNING':
    print('<TR><TD><B>'+env_headline+' (Headline)</B></TD><TD style="background-color:yellow">'+env_value+'</TD></TR>')
  elif env_severity == 'OK':
    print('<TR><TD><B>'+env_headline+' (Headline)</B></TD><TD style="background-color:lightgreen">'+env_value+'</TD></TR>')
  else:
    print('<TR><TD><B>'+env_headline+' (Headline)</B></TD><TD>'+env_value+'</TD></TR>')

if env_column != '':
  print('<TR><TD><B>Table Cell</B></TD><TD>'+env_column+'</TD></TR>')
  if env_severity == 'CRITICAL':
    print('<TR><TD>'+env_rowname+'</TD><TD style="background-color:red">'+env_value+'</TD></TR>')
  elif env_severity == 'WARNING':
    print('<TR><TD>'+env_rowname+'</TD><TD style="background-color:yellow">'+env_value+'</TD></TR>')
  elif env_severity == 'OK':
    print('<TR><TD>'+env_rowname+'</TD><TD style="background-color:lightgreen">'+env_value+'</TD></TR>')
  else:
    print('<TR><TD>'+env_rowname+'</TD><TD>'+env_value+'</TD></TR>')

print('</TABLE>')

# Begin gateway snapshot command
print('<H2>Dataview Snapshot</H2>')

gateway_port = os.environ.get('GatePort', '')
if gateway_port == '':
  print('Please configure GatePort environment variable to retrieve dataview snapshot.')
  sys.exit(0)

url = gateway_hostname+':'+str(gateway_port)+'/rest/snapshot/dataview'

dataviewpath = re.sub('/headlines/cell.*$', '', env_variablepath)
dataviewpath = re.sub('/rows/row.*$', '', dataviewpath)
target = dataviewpath
scope = { "value": True, "severity": True, "snooze": False, "user-assignment": False }
urldata = { "target": target, "scope": scope }

data = json.dumps(urldata).encode('utf-8')

request = urllib.request.Request(url)
base64string = base64.b64encode(bytes('%s:%s' % (gateway_username, gateway_password),'ascii'))
request.add_header("Authorization", "Basic %s" % base64string.decode('utf-8'))

try:
  conn = urllib.request.urlopen(request, data=data, timeout=10)
except urllib.error.URLError as e:
  print('Could not connect the REST Snapshot.<BR>')
  print('URL: '+url+'<BR>')
  print('Dataview: '+dataviewpath+'<BR>')
  print(html.escape(str(e)))
  sys.exit(1)
snapshot = conn.read()                            
conn.close()                                        

json_obj = json.loads(snapshot)
sampletime = json_obj["dataview"]["sample-time"]

print('Sample Time: '+sampletime+'<BR>')

headlines = json_obj["dataview"]["headlines"]
#print(headlines)

print('<H3>Headline</H3>')
print('<TABLE>')
for head in headlines:
  if headlines[head]['severity'] == 'CRITICAL':
    print('<TR><TD><B>'+head+'</B></TD><TD style="background-color:red">'+headlines[head]['value']+'</TD></TR>')
  elif headlines[head]['severity'] == 'WARNING':
    print('<TR><TD><B>'+head+'</B></TD><TD style="background-color:yellow">'+headlines[head]['value']+'</TD></TR>')
  elif headlines[head]['severity'] == 'OK':
    print('<TR><TD><B>'+head+'</B></TD><TD style="background-color:lightgreen">'+headlines[head]['value']+'</TD></TR>')
  else:
    print('<TR><TD><B>'+head+'</B></TD><TD>'+headlines[head]['value']+'</TD></TR>')
print('</TABLE>')

table = json_obj["dataview"]["table"]
#print (table)

print('<H3>Table</H3>')
print('<TABLE>')
print('<TR><TH></TH>', end='')
for row1 in table:
  myrow = row1
for header in table[myrow].items():
  print('<TH>'+header[0]+'</TH>', end='')
print('</TR>')

for row in table:
  print('<TR><TD><B>'+row+'</B></TD>', end='')
  for column in table[row].items():
    if column[1]['severity'] == 'CRITICAL':
      print('<TD style="background-color:red">'+column[1]['value']+'</TD>', end='')
    elif column[1]['severity'] == 'WARNING':
      print('<TD style="background-color:yellow">'+column[1]['value']+'</TD>', end='')
    elif column[1]['severity'] == 'OK':
      print('<TD style="background-color:lightgreen">'+column[1]['value']+'</TD>', end='')
    else:
      print('<TD>'+column[1]['value']+'</TD>', end='')
  print('</TR>')
print('</TABLE>')
